/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 * Contributor: Alessio Pace
 */

package org.objectweb.fractal.cecilia.adl.plugin;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.bindings.JavaBindingUtil;
import org.objectweb.fractal.adl.util.Printer;
import org.objectweb.fractal.api.control.IllegalBindingException;

/**
 * {@link PluginManager} component using a
 * {@link org.objectweb.fractal.adl.FactoryFactory#JAVA_BACKEND java-backend}
 * {@link Factory factory} to load plugin.
 */
public class JavaPluginManager extends AbstractPluginManager {

  @Override
  protected Object createPlugin(final String pluginADL,
      Map<Object, Object> context) throws PluginException {
    if (Printer.isDebugEnable())
      Printer.debug("[plugin-manager] Load new compiler plugin: \'" + pluginADL
          + "\'");

    Object plugin = null;

    try {

      /* XXX reusing the method actual parameter context Map compilation will */
      context = new HashMap<Object, Object>();
      context.put(PLUGIN_FACTORY_BACKEND, "Java");

      plugin = pluginFactoryItf.newComponent(pluginADL, context);

    } catch (final ADLException e) {
      e.printStackTrace();
      throw new PluginException("An error occures while loading the plugin '"
          + pluginADL + "'", e);
    }

    /* if it is a composite component */
    if (plugin instanceof Map) {
      // bind composite plugin to client interface.
      for (final Map.Entry<String, Object> binding : pluginBindings.entrySet()) {
        bindPlugin(binding.getKey(), plugin, binding.getValue());
      }
      // bind plugin to plugin factory.
      bindPlugin(FACTORY, plugin, pluginFactoryItf);
      return ((Map<?, ?>) plugin)
          .get(AbstractPluginManager.PLUGIN_INTERFACE_NAME);
    } else {
      /* else if it is a primitive component */
      // bind primitive plugin to client interface.
      final String[] clientItfs = JavaBindingUtil.listFc(plugin);
      for (final String itfName : clientItfs) {
        final Object pluginBinding = pluginBindings.get(itfName);
        if (pluginBinding == null) continue;
        try {
          JavaBindingUtil.bindFc(plugin, itfName, pluginBinding);
        } catch (final Exception e) {
          throw new PluginException(
              "An error occurs while binding plugin component", e);
        }
      }

      return plugin;
      // end else
    }

  }

  private void bindPlugin(final String itfName, final Object clientComponent,
      final Object server) throws PluginException {
    if (clientComponent instanceof Map) {
      final Map<?, ?> composite = (Map<?, ?>) clientComponent;
      try {
        JavaBindingUtil.bindComposite(composite, itfName, server);
      } catch (final IllegalBindingException e) {
        // ignore
      }
    } else {
      try {
        JavaBindingUtil.bindFc(clientComponent, itfName, server);
      } catch (final Exception e) {
        throw new PluginException(
            "An error occurs while binding plugin component", e);
      }
    }
  }
}
